@@ -16,6 +16,8 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import os
+import sys
 import unittest
 
 from pathlib import Path
@@ -24,6 +26,8 @@ from gvm.connections import DEFAULT_UNIX_SOCKET_PATH, DEFAULT_TIMEOUT
 
 from gvmtools.parser import CliParser
 
+from . import SuppressOutput
+
 __here__ = Path(__file__).parent.resolve()
 
 
@@ -133,32 +137,36 @@ class RootArgumentsParserTest(ParserTestCase):
         self.assertEqual(args.loglevel, 'ERROR')
 
     def test_loglevel_after_subparser(self):
-        with self.assertRaises(SystemExit):
-            self.parser.parse_args(['socket', '--log', 'ERROR'])
+        with SuppressOutput(suppress_stderr=True):
+            with self.assertRaises(SystemExit):
+                self.parser.parse_args(['socket', '--log', 'ERROR'])
 
     def test_timeout(self):
         args = self.parser.parse_args(['--timeout', '1000', 'socket'])
         self.assertEqual(args.timeout, 1000)
 
     def test_timeout_after_subparser(self):
-        with self.assertRaises(SystemExit):
-            self.parser.parse_args(['socket', '--timeout', '1000'])
+        with SuppressOutput(suppress_stderr=True):
+            with self.assertRaises(SystemExit):
+                self.parser.parse_args(['socket', '--timeout', '1000'])
 
     def test_gmp_username(self):
         args = self.parser.parse_args(['--gmp-username', 'foo', 'socket'])
         self.assertEqual(args.gmp_username, 'foo')
 
     def test_gmp_username_after_subparser(self):
-        with self.assertRaises(SystemExit):
-            self.parser.parse_args(['socket', '--gmp-username', 'foo'])
+        with SuppressOutput(suppress_stderr=True):
+            with self.assertRaises(SystemExit):
+                self.parser.parse_args(['socket', '--gmp-username', 'foo'])
 
     def test_gmp_password(self):
         args = self.parser.parse_args(['--gmp-password', 'foo', 'socket'])
         self.assertEqual(args.gmp_password, 'foo')
 
     def test_gmp_password_after_subparser(self):
-        with self.assertRaises(SystemExit):
-            self.parser.parse_args(['socket', '--gmp-password', 'foo'])
+        with SuppressOutput(suppress_stderr=True):
+            with self.assertRaises(SystemExit):
+                self.parser.parse_args(['socket', '--gmp-password', 'foo'])
 
     def test_with_unknown_args(self):
         args, script_args = self.parser.parse_known_args(
@@ -171,8 +179,7 @@ class RootArgumentsParserTest(ParserTestCase):
 class SocketParserTestCase(ParserTestCase):
     def test_defaults(self):
         args = self.parser.parse_args(['socket'])
-        self.assertIsNone(args.sockpath)
-        self.assertEqual(args.socketpath, '/usr/local/var/run/gvmd.sock')
+        self.assertEqual(args.socketpath, DEFAULT_UNIX_SOCKET_PATH)
 
     def test_connection_type(self):
         args = self.parser.parse_args(['socket'])
@@ -180,7 +187,7 @@ class SocketParserTestCase(ParserTestCase):
 
     def test_sockpath(self):
         args = self.parser.parse_args(['socket', '--sockpath', 'foo.sock'])
-        self.assertEqual(args.sockpath, 'foo.sock')
+        self.assertEqual(args.socketpath, 'foo.sock')
 
     def test_socketpath(self):
         args = self.parser.parse_args(['socket', '--socketpath', 'foo.sock'])
@@ -305,15 +312,39 @@ class CustomizeParserTestCase(ParserTestCase):
 class HelpFormattingParserTestCase(ParserTestCase):
     # pylint: disable=protected-access
     maxDiff = None
+    python_version = '.'.join([str(i) for i in sys.version_info[:2]])
 
-    def _snapshot_path(self, name):
+    def setUp(self):
+        super().setUp()
+
+        # ensure all tests are using the same terminal width
+        self.columns = os.environ.get('COLUMNS')
+        os.environ['COLUMNS'] = '80'
+
+    def tearDown(self):
+        super().tearDown()
+
+        if not self.columns:
+            del os.environ['COLUMNS']
+        else:
+            os.environ['COLUMNS'] = self.columns
+
+    def _snapshot_specific_path(self, name):
+        return __here__ / '{}.{}.snap'.format(name, self.python_version)
+
+    def _snapshot_generic_path(self, name):
         return __here__ / '{}.snap'.format(name)
 
-    def _load_snapshot(self, path):
-        return path.read_text()
+    def _snapshot_failed_path(self, name):
+        return __here__ / '{}.{}-failed.snap'.format(name, self.python_version)
 
-    def _write_snapshot(self, path, output):
-        path.write_text(output)
+    def _snapshot_path(self, name):
+        snapshot_specific_path = self._snapshot_specific_path(name)
+
+        if snapshot_specific_path.exists():
+            return snapshot_specific_path
+
+        return self._snapshot_generic_path(name)
 
     def assert_snapshot(self, name, output):
         path = self._snapshot_path(name)
@@ -322,7 +353,15 @@ class HelpFormattingParserTestCase(ParserTestCase):
             path.write_text(output)
 
         content = path.read_text()
-        self.assertEqual(content, output, 'Snapshot differs from output')
+
+        try:
+            self.assertEqual(content, output, 'Snapshot differs from output')
+        except AssertionError:
+            # write new output to snapshot file
+            # reraise error afterwards
+            path = self._snapshot_failed_path(name)
+            path.write_text(output)
+            raise
 
     def test_root_help(self):
         help_output = self.parser._parser.format_help()
@@ -333,9 +372,11 @@ class HelpFormattingParserTestCase(ParserTestCase):
         self.assert_snapshot('socket_help', help_output)
 
     def test_ssh_help(self):
+        self.parser._set_defaults(None)
         help_output = self.parser._parser_ssh.format_help()
         self.assert_snapshot('ssh_help', help_output)
 
     def test_tls_help(self):
+        self.parser._set_defaults(None)
         help_output = self.parser._parser_tls.format_help()
         self.assert_snapshot('tls_help', help_output)
